Module Control
!-------------------------------------------------------------------
! This module contains the variables and subroutines that control
! input, output and error handling. They need to be in one module,
! because some fatal errors require a closing of all open files to save
! as much information as possible.
! ErrorLevel:
!
!  -2 ... do not print severe warnings
!
!  -1 ... do not print warnings
!
!   0 ... print every warning
!
!   1 ... abort in case of a severe warning
!
!   2 ... abort even in case of a warning
! 13.09.2002: including the definition of dp for the determination
!             of the precision used
! 05.01.04: putting all numerical constants here as this file is used
!           in every other module
! 01.03.04: start to build a collection system on error messages,
!           -creating new array i_errors containing the information
!            how often a warning and/or error has been produced
!           -new routines to add 1 in array errors, to clean errors
!            and to write the corresponding numbers 
!-------------------------------------------------------------------

 Interface Is_NaN
  Module Procedure IsNaN_r, IsNaN_v
 End Interface

 Interface FindPosition
  Module Procedure FindPosition_R
 End Interface

! global variables
! double and quadrupole precision
#ifdef QUADRUPOLE
Integer, Parameter :: dp = Selected_real_kind(25,450)
#else
Integer, Parameter :: dp = Selected_real_kind(15,300)
#endif

#ifdef ONLYDOUBLE
Integer, Parameter :: qp = Selected_real_kind(15,300)
#else
Integer, Parameter :: qp = Selected_real_kind(25,450)
#endif

 !------------------------------------
 ! variables for spectrum calculation
 !------------------------------------
 Real(dp) :: delta_mass = 1.e-6_dp    ! precision of mass calculation
 Logical :: WriteOut = .False.        ! write out debug information
 Integer :: n_run = 40         ! maximal number of iterations in mass calculation
 Logical :: GenerationMixing=.False.  ! if (s)fermion generations mix
 !--------------------------------------
 ! variables for process calculations
 !--------------------------------------
 Logical :: L_CS = .False. ! if cross sections should be calculated at all
 Logical :: L_BR=.True.    ! if branching ratios should be calculated at all
 !---------------------------------------------------
 ! for debugging, to get information on the screen
 !---------------------------------------------------
 Logical :: output_screen = .False.
 !--------------------------------------------------------------
 ! if Higgs masses and mixings from external programs are used
 !--------------------------------------------------------------
 Logical :: External_Higgs = .False.
 !--------------------------------------------------------------
 ! if mass spectrum and mixings from external programs are used
 !--------------------------------------------------------------
 Logical :: External_Spectrum = .False.
 !------------------------------------------
 !------------------------------------------
 ! warning and error system
 !------------------------------------------
 Integer, Save :: Iname, ErrorLevel=0, NumberOfOpenFiles=1, ErrCan=10
 Character (Len=40), Save :: NameOfUnit(40)
 !-------------------------
 ! for the error system 
 !-------------------------
 Character(len=60) :: Math_Error(19) =                                &
  & (/ "Routine OdeInt, stepsize smaller than minimum:              " &
  &  , "Routine OdeInt, maximal value > 10^36:                      " &
  &  , "Routine OdeInt, too many steps:                             " &
  &  , "Routine OdeIntB, boundary condition not fullfilled:         " &
  &  , "Routine OdeIntB, stepsize smaller than minimum:             " &
  &  , "Routine OdeIntB, maximal value > 10^36:                     " &
  &  , "Routine OdeIntB, too many steps:                            " &
  &  , "Routine rkqs, stepsize underflow:                           " &
  &  , "Routine ComplexEigenSystem, array dimensions do not match:  " &
  &  , "Routine ComplexEigenSystem, numerical precision not reached:" &
  &  , "Routine RealEigenSystem, array dimensions do not match:     " &
  &  , "Routine RealEigenSystem, numerical precision not reached:   " &
  &  , "Routine tqli, array dimensions do not match:                " &
  &  , "Routine tqli, too many iterations:                          " &
  &  , "Routine Dgauss, too high accuracy required:                 " &
  &  , "Routine DgaussInt, too high accuracy required:              " &
  &  , "Routine Kappa, precision problem:                           " &
  &  , "Routine IntRomb: step size to small:                        " &
  &  , "Routine IntRomb: too many steps:                            " &
  & /)
 Character(len=60) :: SM_Error(2) =                                   &
  & (/ "Routine CalculateRunningMasses: Q_low > m_b(m_b)            " &
  & ,  "Routine CalculateRunningMasses: Max(Q_low,m_b(m_b) > Q_max  " &
  & /)
 Character(len=60) :: SusyM_Error(9) =                                   &
  & (/ "Routine CharginoMass2: diagonalization failed               "    &
  & ,  "Routine TreeMassesMSSM: negative sneutrino mass squared     "    &
  & ,  "Routine CharginoMass2: numerical problem in diagonalization "    &
  & ,  "Routine NeutralinoMass: numerical problem in diagonalization"    &
  & ,  "Routine NeutralinoMass4: diagonalization failed             "    &
  & ,  "ScalarMassMSSMeff: numerical problem in diagonalization     "    &
  & ,  "ScalarMassMSSMeff: encountered a negative mass squared      "    &
  & ,  "Routine ScalarMassMSSMeff: diagonalization failed           "    &
  & ,  "SfermionMass1mssm: encountered a negative mass squared      "    &
  & /)
 Character(len=60) :: InOut_Error(7) =                                   &
  & (/ "Routine LesHouches_Input: unknown error occured:            "    &
  & ,  "Routine LesHouches_Input: Unknown entry for Block MODSEL    "    &
  & ,  "LesHouches_Input: model must be specified before parameters "    &
  & ,  "Routine LesHouches_Input: Unknown entry for Block MINPAR    "    &
  & ,  "LesHouches_Input: model has not been specified completly    "    &
  & ,  "LesHouches_Input: a serious error has been part of the input"    &
  & ,  "LesHouches_Input: Higgs sector has not been fully specified "    &
  & /)
 Character(len=60) :: Sugra_Error(6) =                                   &
  & (/ "Routine BoundaryEW: negative scalar mass as input           "    &
  & ,  "Routine BoundaryEW2: negative scalar mass as input          "    &
  & ,  "Routine RunRGE: entering non-perturbative regime            "    &
  & ,  "Routine RunRGE: g1 and g2 did not meet at the high scale    "    &
  & ,  "Routine Sugra: run did not converge                         "    &
  & ,  "Routine RunRGE: m_nu_R > m_GUT                              "    &
  & /)
 Character(len=60) :: LoopMass_Error(16) =                               &
  & (/ "CharginoMass_Loop: numerical problem in diagonalization     "    &
  & ,  "Routine CharginoMass_Loop: diagonalization failed           "    &
  & ,  "NeutralinoMass_Loop: numerical problem in diagonalization   "    &
  & ,  "Routine NeutralinoMass_Loop: diagonalization failed         "    &
  & ,  "SleptonMass_1L: encountered a negative mass squared         "    &
  & ,  "SleptonMass_1L: p^2 iteration did not converge              "    &
  & ,  "SneutrinoMass_1L: encountered a negative mass squared       "    &
  & ,  "SneutrinoMass_1L: p^2 iteration did not converge            "    &
  & ,  "SquarkMass_1L: encountered a negative mass squared          "    &
  & ,  "SquarkMass_1L: p^2 iteration did not converge               "    &
  & ,  "LoopMassesMSSM: m_h0^2 <0 in the 1-loop calculation         "    &
  & ,  "LoopMassesMSSM: m_H+^2 <0 in the 1-loop calculation         "    &
  & ,  "LoopMassesMSSM: m_A0^2 <0 in the 1-loop calculation         "    &
  & ,  "LoopMassesMSSM: |mu|^2 > 10^20 in the 1-loop calculation    "    &
  & ,  "LoopMassesMSSM: |mu|^2 < 0 in the 1-loop calculation        "    &
  & ,  "LoopMassesMSSM: mZ^2(mZ) < 0                                "    &
  & /)
 Character(len=60) :: TwoLoopHiggs_Error(9) =                            &
  & (/ "PiPseudoScalar2: encountered negative stop mass squared     "    &
  & ,  "PiPseudoScalar2: encountered negative sbottom mass squared  "    &
  & ,  "PiPseudoScalar2: encountered negative stau mass squared     "    &
  & ,  "PiScalar2: encountered negative stop mass squared           "    &
  & ,  "PiScalar2: encountered negative sbottom mass squared        "    &
  & ,  "PiScalar2: encountered negative stau mass squared           "    &
  & ,  "Two_Loop_Tadpoles: encountered negative stop mass squared   "    &
  & ,  "Two_Loop_Tadpoles: encountered negative sbottom mass squared"    &
  & ,  "Two_Loop_Tadpoles: encountered negative stau mass squared   "    &
  & /)

 Integer, Private :: i_errors(1000)=0

!----------------------------
! numerical constants
!----------------------------
! powers and multiples of Pi
!----------------------------
Real(dp), Parameter :: Pi = 3.1415926535897932384626433832795029_dp           &
   & , Pi2 = Pi**2, Pi3 = Pi2*Pi                                              &
   & , pi2o3 = Pi2 / 3._dp, pi2o6 =0.5_dp * Pi2o3, pi2o12 = 0.5_dp * Pi2o6    &
   & , oo3pi = 1._dp / (3._dp * pi), oo4Pi = 1._dp / (4._dp * Pi)             &
   & , oo8Pi = 0.5_dp * oo4Pi, oo16Pi = 0.5_dp * oo8Pi                        &
   & , oo32Pi = 0.5_dp * oo16Pi, oo64Pi = 0.5_dp * oo32Pi                     &
   & , oo48Pi = oo4Pi / 12._dp, fo3Pi = 4._dp / (3._dp * Pi)                  &
   & , oo6pi2 = 1._dp / (6._dp * Pi2), oo8pi2 = 1._dp / (8._dp * Pi2)         &
   & , oo16pi2 = 0.5_dp * oo8pi2, oo64pi2 = 0.25_dp * oo16pi2                 &
   & , oo32pi2 = 0.5_dp * oo16pi2                                             &
   & , oo128pi3 = 1._dp / (128._dp * Pi3)                                     &
   & , oo256pi3 = 1._dp / (256._dp * Pi3), oo512pi3 = 0.5_dp * oo256pi3    
!----------------------------------------------
! special values of Logs
!----------------------------------------------
Real(dp), Parameter :: Log2 = 0.6931471805599453094172321214581765680755_dp
!----------------------------------------------
! special values of the Rieman zeta function
!----------------------------------------------
Real(dp), Parameter :: Zeta2 = Pi2 / 6._dp                    &   
   & , zeta3 = 1.202056903159594285399738161511449990765_dp   &
   & , zeta4 = 1.082323233711138191516003696541167902775_dp   &
   & , zeta5 = 1.036927755143369926331365486457034168057_dp
!----------------------------------------------
! special values of the Gamma function
!----------------------------------------------
 Real(dp), Parameter :: &
   &   Gamma_1o3 = 2.678938534707747633655692940974677644129_dp         &
   & , Gamma_2o3 = 1.354117939426400416945288028154513785519_dp
!---------------------------------------------
! roots and powers of roots
!---------------------------------------------
Real(dp), Parameter :: &
    &   Sqrt2 = 1.41421356237309504880168872420969807856967187537694807317_dp &
    & , ooSqrt2 = 1._dp / Sqrt2                                               &
    & , Sqrt3 = 1.73205080756887729352744634150587236694280525381038062805_dp &
    & , ooSqrt3 = 1._dp / Sqrt3
!------------------------------
! special numbers
!------------------------------
 Complex(dp), Parameter :: ZeroC=(0._dp,0._dp), Ic = (0._dp, 1._dp)
 Real(dp), Parameter :: NearlyZero = 10._dp * Epsilon(1._dp)
!------------------------------
! matrices 
!------------------------------
 Real(dp), Parameter :: id2R(2,2) = &
    & Reshape( Source = (/ 1, 0, 0, 1 /), shape = (/2, 2/) )
 Complex(dp), Parameter :: id2C(2,2) = &
    & Reshape( Source = (/ 1, 0, 0, 1 /), shape = (/2, 2/) )
 Complex(dp), Parameter :: id3C(3,3) = &
    & Reshape( Source = (/ 1, 0, 0, & 
    &                      0, 1, 0, &
    &                      0, 0, 1 /), shape = (/3, 3/) )
 Complex(dp), Parameter :: id4C(4,4) = &
    & Reshape( Source = (/ 1, 0, 0, 0, &
    &                      0, 1, 0, 0, &
    &                      0, 0, 1, 0, &
    &                      0, 0, 0, 1 /), shape = (/4, 4/) )
 Complex(dp), Parameter :: id6C(6,6) = &
    & Reshape( Source = (/ 1, 0, 0, 0, 0, 0, &
    &                      0, 1, 0, 0, 0, 0, &
    &                      0, 0, 1, 0, 0, 0, &
    &                      0, 0, 0, 1, 0, 0, &
    &                      0, 0, 0, 0, 1, 0, &
    &                      0, 0, 0, 0, 0, 1 /), shape = (/6, 6/) )
 Complex(dp), Parameter :: Zero33C(3,3) = ZeroC
! global variables

Contains 


 Subroutine AddError(i)
 !--------------------------------------------------------------------
 ! is used to count number of errors
 ! written by Werner Porod, 01.03.04
 !--------------------------------------------------------------------
 Implicit None
  Integer, intent(in) :: i
  i_Errors(i) = i_Errors(i) + 1
 End Subroutine AddError


 Character Function Bu(n)
 !---------------------------------------------------------------
 ! changes integer n [0,9] into the corresponding character
 !---------------------------------------------------------------
 Implicit None
  Integer, Intent(in) :: n
  Bu = Char(48+n)
 End Function Bu


 Subroutine CheckIndex(i,range,name1,name2)
 !--------------------------------------------------------------------
 ! checks if the index i is between 1 and range 
 ! in case that it is out range, the following actions happen:
 !    ErrorLevel >= 0, termination of program after error message is
 !                     written
 !    ErroLevel = -1, only warning, this can lead to side effects  
 ! written by Werner Porod, 20.9.2000
 !--------------------------------------------------------------------
 Implicit None

  Integer, Intent(In)            :: i,range
  Character (Len=5), Intent(In)  :: name1,name2

  If ((i.Lt.1).Or.(i.Gt.range)) Then
   If (ErrorLevel.Ge.0) Then
    Write (ErrCan,*) 'Severe problem in unit ',NameOfUnit(Iname)
    Write (ErrCan,*) 'Index ',name1,'is out of range: (',name1,name2, &
   &             ') = ',i,range
    Call TerminateProgram
   Else
    Write (ErrCan,*) 'Severe warning from unit ',NameOfUnit(Iname)
    Write (ErrCan,*) 'Index ',name1,'is out of range: (',name1,name2, &
   &             ') = ',i,range
   Endif
  Endif

 End Subroutine CheckIndex


 Subroutine CleanErrors()
 !--------------------------------------------------------------------
 ! is used to reset the array i_errors
 ! written by Werner Porod, 01.03.04
 !--------------------------------------------------------------------
 Implicit None
  i_Errors = 0
 End Subroutine CleanErrors


 Subroutine Closing()
 Implicit None
 Integer :: i1

 Do i1=ErrCan,ErrCan+NumberOfOpenFiles-1
  Close(i1)
 End Do

 End Subroutine Closing


 Subroutine GetError(i)
 !--------------------------------------------------------------------
 ! is used to count number of errors
 ! written by Werner Porod, 01.03.04
 !--------------------------------------------------------------------
 Implicit None
  Integer, intent(out) :: i(size(i_errors))
  i = i_Errors
 End Subroutine GetError


 Subroutine FindPosition_R(n, mat, wert, ii, jj)
 !------------------------------------------------------------------------
 ! finds the position of wert in an n times n matrix
 ! written by Werner Porod, 21.05.2007
 !------------------------------------------------------------------------
 Implicit None
  !-------
  ! input
  !-------
  Integer, Intent(in) :: n         ! dimension of matrix
  Real(dp), Intent(in) :: mat(n,n) ! matrix to be checked
  Real(dp), Intent(in) :: wert     ! value to be looked for
  !--------
  ! output
  !--------
  Integer, Intent(out) :: ii, jj   ! position mat(ii,jj) corresponding to wert
 
  Integer :: i1, i2
 
  Do i1=1,n
   Do i2=1,n
    If (wert.Eq.mat(i1,i2)) Then
     ii = i1
     jj = i2
     Return
    End If
   End Do
  End Do

 End Subroutine FindPosition_R


 Subroutine InitializeControl(io, outfile, name)
 !---------------------------------------------------------------
 ! input:
 !  io ........ channel, to which normal output is written
 !  outfile ... file name connected to channel io
 !  name ...... name of the main program
 !---------------------------------------------------------------
 Implicit None
  Character(len=*), intent(in) :: outfile
  Character(len=*), intent(in) :: name
  Integer, intent(in) :: io
  Iname = 1
  NameOfUnit(Iname) = name

  Open(ErrCan,file="Messages.out",status="unknown")
  Open(io,file=outfile,status="unknown")
  NumberOfOpenFiles = NumberOfOpenFiles + 1


  ErrorLevel = 0
  GenerationMixing=.False.
  L_BR=.True.
  L_CS=.True.

  Open(99,file="Control.in",status="old",err=200)
   Read(99,*) ErrorLevel
   Read(99,*) GenerationMixing
   Read(99,*) L_BR
   Read(99,*) L_CS
  Close(99)

 200  Return

 End Subroutine InitializeControl


 Logical Function IsNaN_r(x)
 !---------------------------------------------------------------------
 ! tests if x is NaN. Comparison of NaN with any number gives FALSE
 !---------------------------------------------------------------------
 implicit none
  Real(dp), Intent(in) :: x

  IsNaN_r = .Not. ((x.Gt.0._dp).Or.(x.Lt.0._dp).Or.(x.Eq.0._dp))

 End Function IsNaN_r


 Logical Function IsNaN_v(x)
 !---------------------------------------------------------------------
 ! tests if at least one element of x is NaN. 
 ! Comparison of NaN with any number gives FALSE
 !---------------------------------------------------------------------
 Real(dp), Intent(in) :: x(:)

 Integer :: l1, i1

 l1 = Size(x)

 IsNaN_v = .False.
 
 Do i1=1,l1
  IsNaN_v = .Not. ((x(i1).Gt.0._dp).Or.(x(i1).Lt.0._dp).Or.(x(i1).Eq.0._dp))
  if (IsNaN_v) exit
 end do

 End Function IsNaN_v

 Subroutine ModelNotIncluded(i1,i2,i3,i4)
 !-------------------------------------------------------------------
 ! States which models are not yet included in a specific subroutine
 ! written by Werner Porod, 20.9.2000
 !-------------------------------------------------------------------
 Implicit None
  
  Integer, Intent(In) :: i1,i2,i3,i4

  If (ErrorLevel.Le.0) Then
   Write (ErrCan,*) 'Warning from unit ',NameOfUnit(Iname)
   If (i1.Ne.0) Write(ErrCan,*) ' MSSM not yet included '
   If (i2.Ne.0) Write(ErrCan,*) ' 1-generation epsilon model not yet included '
   If (i3.Ne.0) Write(ErrCan,*) ' 3-generation epsilon model not yet included '
   If (i4.Ne.0) Write(ErrCan,*) ' full RP model not yet included '
  End If

 End Subroutine ModelNotIncluded


 Subroutine TerminateProgram
 !-----------------------------------------------------------------------
 ! This subroutine terminates a program if a fatal error occurs.
 ! Before doing this, it writes the tree of calling subroutines to
 ! the file which is connected to the channel ErrCan
 ! written by Werner Porod, 20.9.2000
 !-----------------------------------------------------------------------
 Implicit None

  Integer :: i1

  Write (ErrCan,*) "  "
  Write (ErrCan,*) "ErrorLevel, Iname:",ErrorLevel, Iname
  Write (ErrCan,*) &
    & "The error has occured in the following chain of subroutines:"
  Do i1=1,Size(NameOfUnit)
   Write (ErrCan,*) NameOfUnit(i1)
  End Do
  Write (ErrCan,*) "  "
  Write (ErrCan,*) "Hopefully you find the error soon"
  Do i1=ErrCan,ErrCan+NumberOfOpenFiles-1
   Close(i1)
  End Do
  Stop "Subroutine TerminateProgram"

 End Subroutine TerminateProgram


 Subroutine WriteNumberOfErrors(io)
 Implicit None
  Integer, Intent(in) :: io

  Integer :: i1
! module Mathematics
  Do i1=1,100
   If (i_errors(i1).Gt.0) Write(io,*) Trim(Math_Error(i1)),i_errors(i1)
  End Do
! module StandardModel
  Do i1=1,100
   If (i_errors(100+i1).Gt.0) Write(io,*) Trim(SM_Error(i1)),i_errors(100+i1)
  End Do

! module SusyMasses
  Do i1=1,100
   If (i_errors(200+i1).Gt.0) Write(io,*) &
      &    Trim(SusyM_Error(i1)),i_errors(200+i1)
  End Do

! module InputOutput
  Do i1=1,100
   If (i_errors(300+i1).Gt.0) Write(io,*) &
      &    Trim(InOut_Error(i1)),i_errors(300+i1)
  End Do

! module Sugra
  Do i1=1,100
   If (i_errors(400+i1).Gt.0) Write(io,*) &
      &    Trim(Sugra_Error(i1)),i_errors(400+i1)
  End Do

! module LoopMasses
  Do i1=1,50
   If (i_errors(500+i1).Gt.0) Write(io,*) &
      &    Trim(LoopMass_Error(i1)),i_errors(500+i1)
  End Do

! module TwoLoopHiggs
  Do i1=1,50
   If (i_errors(550+i1).Gt.0) Write(io,*) &
      &    Trim(LoopMass_Error(i1)),i_errors(550+i1)
  End Do

 End Subroutine WriteNumberOfErrors


End Module Control

